package javax.swing.plaf.basic;

import javax.swing.Icon;
import java.awt.Insets;
import java.awt.FontMetrics;
import java.awt.Dimension;
import javax.swing.ButtonModel;
import javax.swing.text.View;
import javax.swing.SwingUtilities;
import java.awt.Rectangle;
import javax.swing.AbstractButton;
import java.awt.Graphics;
import sun.awt.AppContext;
import javax.swing.plaf.ComponentUI;
import javax.swing.JComponent;

public class BasicToggleButtonUI extends BasicButtonUI
{
    private static final Object BASIC_TOGGLE_BUTTON_UI_KEY;
    private static final String propertyPrefix = "ToggleButton.";
    
    public BasicToggleButtonUI() {
    }
    
    public static ComponentUI createUI(JComponent component) {
        AppContext appContext = AppContext.getAppContext();
        BasicToggleButtonUI value = (BasicToggleButtonUI)appContext.get(BasicToggleButtonUI.BASIC_TOGGLE_BUTTON_UI_KEY);
        if (value == null) {
            value = new BasicToggleButtonUI();
            appContext.put(BasicToggleButtonUI.BASIC_TOGGLE_BUTTON_UI_KEY, value);
            return value;
        }
        return value;
    }
    
    @Override
    protected String getPropertyPrefix() {
        return "ToggleButton.";
    }
    
    @Override
    public void paint(Graphics graphics, JComponent c) {
        AbstractButton abstractButton = (AbstractButton)c;
        ButtonModel model = abstractButton.getModel();
        Dimension size = abstractButton.getSize();
        FontMetrics fontMetrics = graphics.getFontMetrics();
        Insets insets = c.getInsets();
        Rectangle viewR = new Rectangle(size);
        viewR.x += insets.left;
        viewR.y += insets.top;
        viewR.width -= insets.right + viewR.x;
        viewR.height -= insets.bottom + viewR.y;
        Rectangle iconR = new Rectangle();
        Rectangle textR = new Rectangle();
        graphics.setFont(c.getFont());
        String layoutCompoundLabel = SwingUtilities.layoutCompoundLabel(c, fontMetrics, abstractButton.getText(), abstractButton.getIcon(), abstractButton.getVerticalAlignment(), abstractButton.getHorizontalAlignment(), abstractButton.getVerticalTextPosition(), abstractButton.getHorizontalTextPosition(), viewR, iconR, textR, (abstractButton.getText() == null) ? 0 : abstractButton.getIconTextGap());
        graphics.setColor(abstractButton.getBackground());
        if ((model.isArmed() && model.isPressed()) || model.isSelected()) {
            this.paintButtonPressed(graphics, abstractButton);
        }
        if (abstractButton.getIcon() != null) {
            this.paintIcon(graphics, abstractButton, iconR);
        }
        if (layoutCompoundLabel != null && !layoutCompoundLabel.equals("")) {
            View view = (View)c.getClientProperty("html");
            if (view != null) {
                view.paint(graphics, textR);
            }
            else {
                this.paintText(graphics, abstractButton, textR, layoutCompoundLabel);
            }
        }
        if (abstractButton.isFocusPainted() && abstractButton.hasFocus()) {
            this.paintFocus(graphics, abstractButton, viewR, textR, iconR);
        }
    }
    
    protected void paintIcon(Graphics graphics, AbstractButton abstractButton, Rectangle rectangle) {
        ButtonModel model = abstractButton.getModel();
        Icon icon = null;
        if (!model.isEnabled()) {
            if (model.isSelected()) {
                icon = abstractButton.getDisabledSelectedIcon();
            }
            else {
                icon = abstractButton.getDisabledIcon();
            }
        }
        else if (model.isPressed() && model.isArmed()) {
            icon = abstractButton.getPressedIcon();
            if (icon == null) {
                icon = abstractButton.getSelectedIcon();
            }
        }
        else if (model.isSelected()) {
            if (abstractButton.isRolloverEnabled() && model.isRollover()) {
                icon = abstractButton.getRolloverSelectedIcon();
                if (icon == null) {
                    icon = abstractButton.getSelectedIcon();
                }
            }
            else {
                icon = abstractButton.getSelectedIcon();
            }
        }
        else if (abstractButton.isRolloverEnabled() && model.isRollover()) {
            icon = abstractButton.getRolloverIcon();
        }
        if (icon == null) {
            icon = abstractButton.getIcon();
        }
        icon.paintIcon(abstractButton, graphics, rectangle.x, rectangle.y);
    }
    
    @Override
    protected int getTextShiftOffset() {
        return 0;
    }
    
    static {
        BASIC_TOGGLE_BUTTON_UI_KEY = new Object();
    }
}
